package de.hhhammer.prozessor.core.builder;

import de.hhhammer.prozessor.core.annotation.Builder;

import javax.annotation.processing.Completion;
import javax.annotation.processing.ProcessingEnvironment;
import javax.annotation.processing.Processor;
import javax.annotation.processing.RoundEnvironment;
import javax.lang.model.SourceVersion;
import javax.lang.model.element.*;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

public final class BuilderProcessor implements Processor {

    public BuilderProcessor() {
        System.out.println("Processor created.");
    }

    @Override
    public Set<String> getSupportedOptions() {
        return Set.of();
    }

    @Override
    public Set<String> getSupportedAnnotationTypes() {
        return Set.of(Builder.class.getCanonicalName());
    }

    @Override
    public SourceVersion getSupportedSourceVersion() {
        return SourceVersion.RELEASE_16;
    }

    @Override
    public void init(ProcessingEnvironment processingEnv) {
        System.out.println("Processor initialized.");
    }

    @Override
    public boolean process(Set<? extends TypeElement> annotations, RoundEnvironment roundEnv) {
        System.out.println("Start processing...");
        for (final var annotation : annotations) {
            System.out.println("Annotation: " + annotation.getSimpleName());

            final Set<BuilderAnnotatedClass> builderAnnotatedClasses = roundEnv.getElementsAnnotatedWith(annotation)
                    .stream()
                    .filter(element -> ElementKind.RECORD.equals(element.getKind()))
                    .map(element -> new BuilderAnnotatedClass((TypeElement) element))
                    .collect(Collectors.toSet());
            System.out.println("Found fields:");
            for (final var builderAnnotatedClass : builderAnnotatedClasses) {
                builderAnnotatedClass.getFields().forEach(e -> System.out.println("- " + e.getSimpleName()));
            }

            System.out.println("Processing finished!");
            return false;
        }

        System.out.println("Nothing found to process!");
        return true;
    }

    @Override
    public Iterable<? extends Completion> getCompletions(Element element, AnnotationMirror annotation, ExecutableElement member, String userText) {
        return List.of();
    }

}
